# -*- coding: utf-8 -*-


class BaseConstant:
    domain = "api.sgroup.qq.com"
    sandBoxDomain = "sandbox.api.sgroup.qq.com"
    scheme = "https"


class APIConstant:
    """目前提供的接口的 uri"""

    guildURI = "/guilds/{guild_id}"
    guildMembersURI = "/guilds/{guild_id}/members"
    guildMemberURI = "/guilds/{guild_id}/members/{user_id}"

    rolesURI = "/guilds/{guild_id}/roles"
    roleURI = "/guilds/{guild_id}/roles/{role_id}"
    memberRoleURI = "/guilds/{guild_id}/members/{user_id}/roles/{role_id}"

    channelsURI = "/guilds/{guild_id}/channels"
    channelURI = "/channels/{channel_id}"
    channelPermissionsURI = "/channels/{channel_id}/members/{user_id}/permissions"
    channelRolePermissionsURI = "/channels/{channel_id}/roles/{role_id}/permissions"

    messagesURI = "/channels/{channel_id}/messages"
    messageURI = "/channels/{channel_id}/messages/{message_id}"

    userMeURI = "/users/@me"
    userMeGuildsURI = "/users/@me/guilds"
    userMeDMURI = "/users/@me/dms"

    gatewayURI = "/gateway"
    gatewayBotURI = "/gateway/bot"

    audioControlURI = "/channels/{channel_id}/audio"
    dmsURI = "/dms/{guild_id}/messages"

    guildMuteURI = "/guilds/{guild_id}/mute"
    guildMemberMuteURI = "/guilds/{guild_id}/members/{user_id}/mute"

    guildAnnounceURI = "/guilds/{guild_id}/announces"
    deleteGuildAnnounceURI = "/guilds/{guild_id}/announces/{message_id}"
    channelAnnounceURI = "/channels/{channel_id}/announces"
    deleteChannelAnnounceURI = "/channels/{channel_id}/announces/{message_id}"

    channelSchedulesURI = "/channels/{channel_id}/schedules"
    channelSchedulesIdURI = "/channels/{channel_id}/schedules/{schedule_id}"


def get_url(url_format, is_sandbox):
    d = BaseConstant.domain
    if is_sandbox:
        d = BaseConstant.sandBoxDomain
    s__format = "{}://{}{}".format(BaseConstant.scheme, d, url_format)
    return s__format



# -*- coding: utf-8 -*-

from .api import (
    listen_events,
    GuildAPI,
    GuildRoleAPI,
    GuildMemberAPI,
    ChannelAPI,
    ChannelPermissionsAPI,
    UserAPI,
    AudioAPI,
    MessageAPI,
    WebsocketAPI,
    MuteAPI,
    AnnouncesAPI,
    DmsAPI,
)

from .async_api import (
    async_listen_events,
    AsyncGuildAPI,
    AsyncGuildRoleAPI,
    AsyncGuildMemberAPI,
    AsyncChannelAPI,
    AsyncChannelPermissionsAPI,
    AsyncUserAPI,
    AsyncAudioAPI,
    AsyncMessageAPI,
    AsyncWebsocketAPI,
    AsyncMuteAPI,
    AsyncAnnouncesAPI,
    AsyncDmsAPI,
)
from .core.network.ws.ws_intents_handler import HandlerType, Handler
from .core.util import logging
from .model import *

logger = logging.getLogger()



# -*- coding: utf-8 -*-

from enum import Enum


class OpCode(Enum):
    """
    CODE	名称	客户端操作	描述
    0	Dispatch	Receive	服务端进行消息推送
    1	Heartbeat	Send/Receive	客户端或服务端发送心跳
    2	Identify	Send	客户端发送鉴权
    6	Resume	Send	客户端恢复连接
    7	Reconnect	Receive	服务端通知客户端重新连接
    9	Invalid Session	Receive	当identify或resume的时候，如果参数有错，服务端会返回该消息
    10	Hello	Receive	当客户端与网关建立ws连接之后，网关下发的第一条消息
    11	Heartbeat ACK	Receive	当发送心跳成功之后，就会收到该消息
    """

    WS_DISPATCH_EVENT = 0
    WS_HEARTBEAT = 1
    WS_IDENTITY = 2
    WS_RESUME = 6
    WS_RECONNECT = 7
    WS_INVALID_SESSION = 9
    WS_HELLO = 10
    WS_HEARTBEAT_ACK = 11



# -*- coding: utf-8 -*-

from enum import Enum


class Intents(Enum):
    INTENT_GUILDS = 1 << 0
    # - GUILD_CREATE // 当机器人加入新guild时
    # - GUILD_UPDATE // 当guild资料发生变更时
    # - GUILD_DELETE // 当机器人退出guild时
    # - CHANNEL_CREATE // 当channel被创建时
    # - CHANNEL_UPDATE // 当channel被更新时
    # - CHANNEL_DELETE // 当channel被删除时

    INTENT_GUILD_MEMBERS = 1 << 1
    # - GUILD_MEMBER_ADD // 当成员加入时
    # - GUILD_MEMBER_UPDATE // 当成员资料变更时
    # - GUILD_MEMBER_REMOVE // 当成员被移除时

    INTENT_GUILD_MESSAGES = 1 << 9

    INTENT_DIRECT_MESSAGE = 1 << 12
    # - DIRECT_MESSAGE_CREATE // 当收到用户发给机器人的私信消息时

    INTENT_AUDIO = 1 << 29
    # - AUDIO_START           // 音频开始播放时
    # - AUDIO_FINISH          // 音频播放结束时
    # - AUDIO_ON_MIC          // 上麦时
    # - AUDIO_OFF_MIC         // 下麦时

    INTENT_GUILD_AT_MESSAGE = 1 << 30
    # - AT_MESSAGE_CREATE // 当收到 @ 机器人的消息时



# -*- coding: utf-8 -*-


class User:
    def __init__(self, data=None):
        self.id: str = ""
        self.username: str = ""
        self.avatar: str = ""
        self.bot: bool = False
        self.union_openid: str = ""
        self.union_user_account: str = ""
        if data is not None:
            self.__dict__ = data


class Member:
    def __init__(self, data=None):
        self.user: User = User()
        self.nick: str = ""
        self.roles = [""]
        self.joined_at: str = ""
        if data is not None:
            self.__dict__ = data


class MemberWithGuildID:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.user: User = User()
        self.nick: str = ""
        self.roles: [str] = [""]
        self.joined_at: str = ""
        if data is not None:
            self.__dict__ = data



# -*- coding: utf-8 -*-
from typing import List

from qqbot.model.guild_role import Role
from qqbot.model.member import User


class GuildMember:
    def __init__(self, data=None):
        self.user: [User] = None
        self.nick: str = ""
        self.roles: List[Role] = []
        self.joined_at: str = ""
        if data is not None:
            self.__dict__ = data


class QueryParams:
    def __init__(self, after: str, limit: int):
        self.after: str = after
        self.limit: int = limit



# -*- coding: utf-8 -*-


class Guild:
    def __init__(self, data=None):
        self.id: str = ""
        self.name: str = ""
        self.icon: str = ""
        self.owner_id: str = ""
        self.owner: bool = False
        self.member_count: int = 0
        self.max_members: int = 0
        self.description: str = ""
        self.joined_at: str = ""
        if data:
            self.__dict__ = data



# -*- coding: utf-8 -*-


class ChannelPermissions:
    def __init__(self, data=None):
        self.channel_id: str = ""
        self.user_id: str = ""
        self.permissions: str = ""
        self.role_id: str = ""
        if data is not None:
            self.__dict__ = data


class UpdatePermission:
    def __init__(self, add: str = None, remove: str = None):
        """
        要求操作人具有管理子频道的权限，如果是机器人，则需要将机器人设置为管理员。
        参数包括add和remove两个字段，分别表示授予的权限以及删除的权限。要授予用户权限即把add对应位置1，删除用户权限即把remove对应位置1。当两个字段同一位都为1，表现为删除权限。
        本接口不支持修改可管理子频道权限。

        :param add: 字符串形式的位图表示赋予用户的权限
        :param remove:字符串形式的位图表示删除用户的权限
        """
        self.add = add
        self.remove = remove



# -*- coding: utf-8 -*-


class STATUS:
    START = 0
    PAUSE = 1
    RESUME = 2
    STOP = 3


class AudioControl:
    def __init__(self, audio_url: str, text: str, status: STATUS):
        self.audio_url = audio_url
        self.text = text
        self.status = status


class AudioAction:
    def __init__(self, data=None):
        self.guild_id: str = ""
        self.channel_id: str = ""
        self.audio_url: str = ""
        self.text: str = ""
        if data is not None:
            self.__dict__ = data



# -*- coding: utf-8 -*-


class WebsocketError:
    CodeInvalidSession = 9001
    CodeConnCloseErr = 9005


class AuthenticationFailedError(RuntimeError):
    def __init__(self, msg):
        self.msgs = msg

    def __str__(self):
        return self.msgs


class NotFoundError(RuntimeError):
    def __init__(self, msg):
        self.msgs = msg

    def __str__(self):
        return self.msgs


class MethodNotAllowedError(RuntimeError):
    def __init__(self, msg):
        self.msgs = msg

    def __str__(self):
        return self.msgs


class SequenceNumberError(RuntimeError):
    def __init__(self, msg):
        self.msgs = msg

    def __str__(self):
        return self.msgs


class ServerError(RuntimeError):
    def __init__(self, msg):
        self.msgs = msg

    def __str__(self):
        return self.msgs



# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages

setup(
    name="qq-bot",
    version=os.getenv("VERSION_NAME"),
    author="veehou",
    author_email="veehou@tencent.com",
    description="qq guild robot sdk with python3, support asyncio and synchronous",
    long_description=open("README.rst").read(),
    # 项目主页
    url="https://github.com/tencent-connect/botpy",
    # 你要安装的包，通过 setuptools.find_packages 找到当前目录下有哪些包
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    # 执照
    license="Tencent",
    # 安装依赖
    install_requires=["requests", "websocket-client", "aiohttp>=3.6.0,<3.8.0"],
    # 分类
    classifiers=[
        # 发展时期,常见的如下
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        # 开发的目标用户
        "Intended Audience :: Developers",
        # 属于什么类型
        "Topic :: Software Development",
        # 许可证信息
        "License :: OSI Approved :: MIT License",
        # 目标 Python 版本
        "Programming Language :: Python :: 3.7",
    ],
)



class DefaultHandler:
    """
    持有handler的实例
    """

    plain = None
    guild = None
    guild_member = None
    channel = None
    message = None
    at_message = None
    direct_message = None
    audio = None
